<?php

namespace Drupal\dismissible_message_bar\Plugin\Block;

use Drupal\Core\Annotation\Translation;
use Drupal\Core\Block\Annotation\Block;
use Drupal\Core\Block\BlockBase;
use Drupal\Core\Plugin\ContainerFactoryPluginInterface;
use Symfony\Component\DependencyInjection\ContainerInterface;
use Symfony\Component\HttpFoundation\Request;

/**
 * Provides a 'DmbNotificationsBlock' block.
 *
 * @Block(
 *  id = "dmb_notifications_block",
 *  admin_label = @Translation("DMB Notifications block"),
 * )
 */
class DmbNotificationsBlock extends BlockBase implements ContainerFactoryPluginInterface {

  /**
   * The current request.
   *
   * @var \Symfony\Component\HttpFoundation\Request
   */
  protected $request;

  /**
   * Creates a DmbNotificationsBlock instance.
   *
   * @param array $configuration
   *   A configuration array containing information about the plugin instance.
   * @param string $plugin_id
   *   The plugin_id for the plugin instance.
   * @param mixed $plugin_definition
   *   The plugin implementation definition.
   * @param \Symfony\Component\HttpFoundation\Request $request
   *   The current request.
   */
  public function __construct(array $configuration, $plugin_id, $plugin_definition, Request $request) {
    parent::__construct($configuration, $plugin_id, $plugin_definition);

    $this->request = $request;
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container, array $configuration, $plugin_id, $plugin_definition) {
    return new static(
      $configuration,
      $plugin_id,
      $plugin_definition,
      $container->get('request_stack')->getCurrentRequest()
    );
  }

  /**
   * {@inheritdoc}
   */
  public function build() {

    // Since LazyBuilder approach does not bypass Varnish, only placeholder is
    // rendered here and populated later through an AJAX call.
    $build = [
      '#type' => 'container',
      'dmb_notifications' => [
        '#type' => 'html_tag',
        '#tag' => 'div',
        '#attributes' => [
          'class' => [
            'dmb-notifications-ajax-wrapper',
          ],
        ],
      ],
      '#attached' => [
        'library' => [
          'dismissible_message_bar/dismissible_message_bar',
        ],
      ],
    ];

    $build['#attached']['drupalSettings']['dmbNotifications']['notificationWrapper'] = '.dmb-notifications-ajax-wrapper';
    $build['#attached']['drupalSettings']['dmbNotifications']['contentType'] = $this->getCurrentContentType($this->request);
    $build['#attached']['drupalSettings']['dmbNotifications']['contentPath'] = $this->request->getPathInfo();

    return $build;
  }

  /**
   * {@inheritdoc}
   */
  public function getCacheContexts() {
    $contexts = parent::getCacheContexts();
    $contexts[] = 'url.path';
    $contexts[] = 'cookies:dismissible_message_bar';

    return $contexts;
  }

  /**
   * Get the current content type.
   *
   * @param \Symfony\Component\HttpFoundation\Request $request
   *   Current Request,
   *
   * @return string
   *   Content type name or 'none'.
   */
  public function getCurrentContentType(Request $request) {
    // Grab the node associated with this content
    $node = $request->attributes->get('node');
    if (empty($node) || !is_object($node)) {
      // Set the "no content type" to 'none' so we don't have empty collision.
      $content_type = FALSE;
    }
    else {
      $content_type = $node->getType();
    }

    return $content_type;
  }

}
